use super::AlterableField::{self, Login, Name, Pass};
use crate::{change_state, prelude::*};
use cryptography::account::Cipher;
use tokio::task::spawn_blocking;

#[inline]
async fn update_account(
    user_id: u64,
    db: &Pool,
    name: String,
    field: AlterableField,
    field_value: String,
    master_pass: String,
) -> crate::Result<()> {
    if field == Name {
        Account::update_name(user_id, &name, &field_value, db).await?;

        return Ok(());
    }

    let salt = Account::get_salt(user_id, &name, db).await?.unwrap();

    let field_value = spawn_blocking(move || {
        let cipher = Cipher::new(master_pass.as_bytes(), &salt);
        let mut field = field_value.into_bytes();
        cipher.encrypt(&mut field);
        field
    })
    .await?;

    match field {
        Login => Account::update_login(user_id, &name, field_value, db).await?,
        Pass => Account::update_password(user_id, &name, field_value, db).await?,
        Name => unreachable!(),
    }

    Ok(())
}

#[inline]
#[allow(clippy::too_many_arguments)]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    locale: LocaleRef,
    name: String,
    field: AlterableField,
    field_value: String,
    master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;

    update_account(user_id, &db, name, field, field_value, master_pass).await?;

    ids.alter_message(
        &bot,
        locale.success_choose_account_to_view.as_ref(),
        menu_markup("get", user_id, &db).await?,
        None,
    )
    .await?;

    Ok(())
}

handler!(get_field(name:String, field:AlterableField, field_value:String),send_master_password, State::GetMasterPass, get_master_pass);

#[inline]
pub async fn alter(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: Pool,
    dialogue: MainDialogue,
    locale: LocaleRef,
    (hash, field): (super::NameHash, AlterableField),
) -> crate::Result<()> {
    let user_id = q.from.id.0;
    let mut ids: MessageIds = q.message.as_ref().unwrap().into();

    let Some(name) = Account::get_name_by_hash(user_id, &hash, &db).await? else {
        bot.send_message(ids.0, "Account wasn't found")
            .reply_markup(deletion_markup(locale))
            .await?;
        bot.answer_callback_query(q.id).await?;
        return Ok(());
    };

    let text = match field {
        Name => {
            change_state!(dialogue, ids, (name, field), State::GetNewName, get_field);
            locale.send_new_name.as_ref()
        }
        Login => {
            change_state!(dialogue, ids, (name, field), State::GetLogin, get_field);
            locale.send_new_login.as_ref()
        }
        Pass => {
            change_state!(dialogue, ids, (name, field), State::GetPassword, get_field);
            locale.send_new_password.as_ref()
        }
    };

    ids.alter_message(&bot, text, None, None).await?;
    bot.answer_callback_query(q.id).await?;

    Ok(())
}
