use crate::{locales::LocaleTypeExt, prelude::*};
use entity::locale::LocaleType;

#[inline]
pub async fn change_locale(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: Pool,
    mut locale: LocaleRef,
    new_locale: LocaleType,
) -> crate::Result<()> {
    let mut ids: MessageIds = q.message.as_ref().unwrap().into();
    let user_id = q.from.id.0;

    let is_successful = new_locale
        .update(user_id, &db)
        .await
        .inspect_err(|err| log::error!("{err}"))
        .unwrap_or(false);

    if !is_successful {
        ids.alter_message(
            &bot,
            locale.something_went_wrong.as_ref(),
            deletion_markup(locale),
            None,
        )
        .await?;
        return Ok(());
    }

    locale = new_locale.get_locale();
    ids.alter_message(
        &bot,
        locale.choose_language.as_ref(),
        language_markup(),
        None,
    )
    .await?;

    Ok(())
}
