use crate::{change_state, prelude::*};
use teloxide::types::ParseMode;
use tokio::task::spawn_blocking;

#[inline]
#[allow(clippy::too_many_arguments)]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    locale: LocaleRef,
    name: String,
    master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;

    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;

    let Some(account) = Account::get(user_id, &name, &db).await? else {
        bot.send_message(msg.chat.id, locale.no_accounts_found.as_ref())
            .reply_markup(deletion_markup(locale))
            .await?;
        return Ok(());
    };

    let account =
        spawn_blocking(move || DecryptedAccount::from_account(account, &master_pass)).await??;

    let text = locale.show_account(&account.name, &account.login, &account.password);

    ids.alter_message(
        &bot,
        text,
        account_markup(&name, false, locale),
        ParseMode::MarkdownV2,
    )
    .await?;
    Ok(())
}

#[inline]
pub async fn decrypt(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: Pool,
    dialogue: MainDialogue,
    locale: LocaleRef,
    hash: super::NameHash,
) -> crate::Result<()> {
    let mut ids: MessageIds = q.message.as_ref().unwrap().into();
    let user_id = q.from.id.0;

    let Some(name) = Account::get_name_by_hash(user_id, &hash, &db).await? else {
        bot.send_message(ids.0, locale.no_accounts_found.as_ref())
            .reply_markup(deletion_markup(locale))
            .await?;
        bot.answer_callback_query(q.id).await?;
        return Ok(());
    };

    ids.alter_message(&bot, locale.send_master_password.as_ref(), None, None)
        .await?;
    bot.answer_callback_query(q.id).await?;

    change_state!(dialogue, ids, (name), State::GetMasterPass, get_master_pass);

    Ok(())
}
