use crate::{change_state, prelude::*};

#[inline]
#[allow(clippy::too_many_arguments)]
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    dialogue: MainDialogue,
    mut ids: MessageIds,
    locale: LocaleRef,
    name: String,
    _: String,
) -> crate::Result<()> {
    dialogue.exit().await?;

    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    Account::delete(user_id, &name, &db).await?;

    ids.alter_message(&bot, locale.success.as_ref(), deletion_markup(locale), None)
        .await?;

    Ok(())
}

#[inline]
pub async fn delete(
    bot: Throttle<Bot>,
    q: CallbackQuery,
    db: Pool,
    dialogue: MainDialogue,
    locale: LocaleRef,
    (hash, is_command): (super::NameHash, bool),
) -> crate::Result<()> {
    let text = locale.send_master_pass_to_delete_account.as_ref();

    let mut ids: MessageIds = q.message.as_ref().unwrap().into();
    let user_id = q.from.id.0;

    let Some(name) = Account::get_name_by_hash(user_id, &hash, &db).await? else {
        bot.send_message(ids.0, locale.no_accounts_found.as_ref())
            .reply_markup(deletion_markup(locale))
            .await?;
        bot.answer_callback_query(q.id).await?;
        return Ok(());
    };

    if is_command {
        ids.alter_message(&bot, text, None, None).await?;
    } else {
        let msg = bot.send_message(ids.0, text).await?;
        ids = MessageIds::from(&msg);
    };

    bot.answer_callback_query(q.id).await?;

    change_state!(dialogue, ids, (name), State::GetMasterPass, get_master_pass);

    Ok(())
}
