use crate::prelude::*;
use std::fmt::Write;
use teloxide::types::ParseMode;

/// Handles /`get_accounts` command by sending the list of copyable account names to the user
#[inline]
pub async fn get_accounts(
    bot: Throttle<Bot>,
    msg: Message,
    db: Pool,
    locale: LocaleRef,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let mut account_names = Account::get_names(user_id, &db);

    let Some(mut text) = account_names.try_next().await? else {
        bot.send_message(msg.chat.id, locale.no_accounts_found.as_ref())
            .reply_markup(deletion_markup(locale))
            .await?;
        return Ok(());
    };

    text.insert(0, '`');
    text.push('`');

    while let Some(name) = account_names.try_next().await? {
        write!(text, "\n`{name}`")?;
    }

    bot.send_message(msg.chat.id, text)
        .parse_mode(ParseMode::MarkdownV2)
        .reply_markup(deletion_markup(locale))
        .await?;
    Ok(())
}
