#[macro_export]
macro_rules! change_state {
    ($dialogue: expr, $previous: expr, ($($param: ident),*), $next_state: expr, $next_func: ident) => {{
        $dialogue
            .update($next_state(Handler::new(
                move |bot, msg, db, dialogue, locale, ids, param| Box::pin($next_func(bot, msg, db, dialogue, locale, ids, $($param,)* param)),
                $previous,
            )))
            .await?;
    }};
}

#[macro_export]
macro_rules! first_handler {
    ($function_name: ident, $message: ident, $next_state: expr, $next_func: ident) => {
        #[inline]
        pub async fn $function_name(
            bot: Throttle<Bot>,
            msg: Message,
            dialogue: MainDialogue,
            locale: LocaleRef,
        ) -> $crate::Result<()> {
            let previous = bot
                .send_message(msg.chat.id, locale.$message.as_ref())
                .await?;

            $crate::change_state!(dialogue, &previous, (), $next_state, $next_func);

            Ok(())
        }
    };
}

#[macro_export]
macro_rules! handler {
    ($function_name: ident ($($param: ident: $type: ty),*), $message: ident, $next_state: expr, $next_func: ident) => {
        #[allow(clippy::too_many_arguments)]
        #[inline]
        async fn $function_name(
            bot: Throttle<Bot>,
            _: Message,
            _: Pool,
            dialogue: MainDialogue,
            mut ids: MessageIds,
            locale: LocaleRef,
            $($param: $type),*
        ) -> $crate::Result<()> {
            ids.alter_message(&bot, locale.$message.as_ref(), None, None).await?;

            $crate::change_state!(dialogue, ids, ($($param),*), $next_state, $next_func);

            Ok(())
        }
    };
}

#[macro_export]
macro_rules! simple_state_handler {
    ($function_name: ident, $check: ident) => {
        #[inline]
        pub async fn $function_name(
            bot: Throttle<Bot>,
            msg: Message,
            db: Pool,
            dialogue: MainDialogue,
            locale: LocaleRef,
            next: PackagedHandler<String>,
        ) -> $crate::Result<()> {
            super::generic::generic(
                bot,
                msg,
                db,
                dialogue,
                |msg, db, locale, param| Box::pin($check(msg, db, locale, param)),
                locale,
                next,
            )
            .await
        }
    };
}

#[macro_export]
macro_rules! export_handlers {
    ($($name: ident),*) => {
        $(
        mod $name;
        pub use $name::$name;
        )
        *
    };
}
