use crate::prelude::*;
use cryptography::passwords::{check_master_pass, PasswordValidity};
use std::fmt::Write as _;

#[inline]
fn process_validity(validity: PasswordValidity, locale: LocaleRef) -> Result<(), String> {
    if validity.is_empty() {
        return Ok(());
    }

    let mut error_text = locale.master_password_too_weak.as_ref().to_owned();

    if validity.contains(PasswordValidity::NO_LOWERCASE) {
        write!(error_text, "\n* {}", locale.no_lowercase).unwrap();
    }
    if validity.contains(PasswordValidity::NO_UPPERCASE) {
        write!(error_text, "\n* {}", locale.no_uppercase).unwrap();
    }
    if validity.contains(PasswordValidity::NO_NUMBER) {
        write!(error_text, "\n* {}", locale.no_numbers).unwrap();
    }
    if validity.contains(PasswordValidity::NO_SPECIAL_CHARACTER) {
        write!(error_text, "\n* {}", locale.no_special_characters).unwrap();
    }
    if validity.contains(PasswordValidity::TOO_SHORT) {
        write!(error_text, "\n* {}", locale.master_pass_too_short).unwrap();
    }

    error_text.push_str(&locale.change_master_password_and_send_again);

    Err(error_text)
}

/// Checks that the account with that name exists
#[inline]
async fn check_new_master_pass(
    _: &Message,
    _: &Pool,
    locale: LocaleRef,
    password: &str,
) -> crate::Result<Option<String>> {
    let validity = check_master_pass(password);

    Ok(process_validity(validity, locale).err())
}

crate::simple_state_handler!(get_new_master_pass, check_new_master_pass);
