use crate::prelude::*;

/// Validates a new account
#[inline]
async fn check_new_account_name(
    msg: &Message,
    db: &Pool,
    locale: LocaleRef,
    name: &str,
) -> crate::Result<Option<String>> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;

    if Account::exists(user_id, name, db).await? {
        Ok(Some(locale.account_already_exists.as_ref().into()))
    } else if !validate_field(name) {
        Ok(Some(locale.invalid_name.as_ref().into()))
    } else {
        Ok(None)
    }
}

crate::simple_state_handler!(get_new_name, check_new_account_name);
